<?php

namespace Crm\News\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Crm\News\Forms\News as NewsForm;
use Crm\News\Http\Requests\NewsResourceRequest;
use Crm\News\Http\Resources\NewsResource;
use Crm\News\Http\Resources\NewsCollection;
use Crm\News\Models\News;
use Crm\News\Actions\NewsAction;
use Crm\News\Actions\NewsActions;


/**
 * Resource controller class for news.
 */
class NewsResourceController extends BaseController
{

    /**
     * Initialize news resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = NewsForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('crm.news.modules'), 'news', guard_url('news'));
            return $next($request);
        });
    }

    /**
     * Display a list of news.
     *
     * @return Response
     */
    public function index(NewsResourceRequest $request)
    {
        $request = $request->all();
        $page = NewsActions::run('paginate', $request);

        $data = new NewsCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('news::news.names'))
            ->view('news::news.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display news.
     *
     * @param Request $request
     * @param Model   $news
     *
     * @return Response
     */
    public function show(NewsResourceRequest $request, News $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new NewsResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('news::news.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('news::news.show')
            ->output();
    }

    /**
     * Show the form for creating a new news.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(NewsResourceRequest $request, News $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new NewsResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('news::news.name'))
            ->view('news::news.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new news.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(NewsResourceRequest $request, News $model)
    {
        try {
            $request = $request->all();
            $model = NewsAction::run('store', $model, $request);
            $data = new NewsResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('news::news.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('news/news/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/news/news'))
                ->redirect();
        }

    }

    /**
     * Show news for editing.
     *
     * @param Request $request
     * @param Model   $news
     *
     * @return Response
     */
    public function edit(NewsResourceRequest $request, News $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new NewsResource($model);
        // return view('news::news.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('news::news.name'))
            ->view('news::news.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the news.
     *
     * @param Request $request
     * @param Model   $news
     *
     * @return Response
     */
    public function update(NewsResourceRequest $request, News $model)
    {
        try {
            $request = $request->all();
            $model = NewsAction::run('update', $model, $request);
            $data = new NewsResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('news::news.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('news/news/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('news/news/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the news.
     *
     * @param Model   $news
     *
     * @return Response
     */
    public function destroy(NewsResourceRequest $request, News $model)
    {
        try {

            $request = $request->all();
            $model = NewsAction::run('destroy', $model, $request);
            $data = new NewsResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('news::news.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('news/news/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('news/news/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
